filterTotalReport <- function (
		inputData,
		scoreCut = 12,
		save = TRUE,
		folder,
		output = "filteredReport") {
		
	###########################################
	########## Check if CSV with dialog #######
###########################################
isCSVdlg <- function(titleMSG, errorMSG) {
		t = 0
		while (t == 0) {
			checkIfCsv <- dlgOpen(title = titleMSG, multiple = FALSE)$res
			checkIfCsv2 <- basename(checkIfCsv)
			checkIfCsv3 <- unlist(strsplit(checkIfCsv2, "\\."))
			checkIfCsv4 <- checkIfCsv3[length(checkIfCsv3)]
			if (toupper(checkIfCsv4) == "CSV") {
				t = 1
				return(checkIfCsv)
			} else {
				dlgMessage(errorMSG)
			}
		}
	}
	###########################################
	########## Check if CSV ###################
###########################################
isCSV <- function(pathFile, errorMSG) {
		t = 0
		checkIfCsv <- pathFile
		checkIfCsv2 <- basename(checkIfCsv)
		checkIfCsv3 <- unlist(strsplit(checkIfCsv2, "\\."))
		checkIfCsv4 <- checkIfCsv3[length(checkIfCsv3)]
		if (toupper(checkIfCsv4) == "CSV") {
			t = 1
			return(t)
		} else {
			return(t)
		}
	}		
	###########################################
	########## Check the input data ###########
###########################################   
if (missing(inputData)) {
		inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
		totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
		message("Input file loaded...")
	} else {
		if (is.data.frame(inputData) == TRUE) {
			totalReport <- inputData
			message("Data frame loaded...")
		} else {
			if (is.character(inputData)) {
				checkIfCsv <- isCSV(inputData)
				if (checkIfCsv == 1) {
					inputTest <- file.access(inputData, 0)
					if (inputTest == 0) {
						totalReport = read.csv(inputData, colClasses = "character", check.names = FALSE)
					} else {
						dlgMessage("The input file specified is not accessible. Please, choose a valid CSV file to be used as input data.")
						inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
						totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
						message("Input file loaded...")
					}
				} else {
					dlgMessage("The input file specified is not in CSV format. Please, choose a valid CSV file to be used as input data.")
					inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
					totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
					message("Input file loaded...")
				}
			} else {
				dlgMessage("The path to the input data must be specified as character string. Please, choose a valid CSV file to be used as input data.")
				inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
				totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
				message("Input file loaded...")
			}
		}
	}
	#### Check the format of the input data and convert to the right format ####
	if (length(grep("RT", names(totalReport)[2])) > 0 && length(grep("DiffRT", names(totalReport)[3])) > 0){
		message("Converting inputData to the format of data(exampleMatchSpectraInverted)[[1]]...")
		totalReport <- data.frame(t(totalReport), stringsAsFactors = FALSE, check.names = FALSE)
		names(totalReport) <- totalReport[1,]
		totalReport <- totalReport[-1,]
		totalReport <- cbind(Name = as.character(row.names(totalReport)), totalReport, stringsAsFactors = F)
		row.names(totalReport) <- 1:nrow(totalReport)
		message("Done!")
	}
	if (length(grep("RT", row.names(totalReport))) > 0 && length(grep("DiffRT", row.names(totalReport))) > 0){
		totalReport <- cbind(Name = as.character(row.names(totalReport)), totalReport, stringsAsFactors = F)
		row.names(totalReport) <- 1:nrow(totalReport)
	}
	###### Remove intesnities below cutOff #####
	removeInt <- function(data, cutOff){
		scores <- data[seq(3,length(data), 4)]
		intReport <- data[seq(4,length(data), 4)]
		suppressWarnings(intReport[as.numeric(scores) < cutOff] <- NA)
		data[seq(4,length(data), 4)] <- intReport
		return(data)
	}
	totalReport <- data.frame(apply(totalReport, 2, function(x) removeInt(x, scoreCut)), stringsAsFactors = FALSE, check.names = FALSE)
	names(totalReport)[1] <- "Name"
	###### Remove compounds with NA everywhere #####
	nSamples <- nrow(totalReport)/4
	findNAs <- function(data){
		intReport <- data[seq(4,length(data), 4)]
		nNAs <- sum(is.na(intReport))
 		return(nNAs)
	}
	nasFound <- as.numeric(unlist(apply(totalReport, 2, function(x) findNAs(x))))
	nasFound <- which(nasFound == nSamples)
	if (length(nasFound) > 0){
		totalReport <- totalReport[-c(nasFound)]
	}
	bigData <- totalReport
	smallData <- totalReport[seq(4,nrow(totalReport), 4),]
	##################################################################
	############### Set folder if SAVE == TRUE #######################
	##################################################################
	if (save) {
		if (missing(folder)) {
			folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
		} else {
			if (is.character(folder)) {
				isFolder <- file.access(as.character(folder), 0)
				if (isFolder == 0) {
					isFolder <- file.info(folder)
					if (isFolder$isdir != TRUE) {
						message("The folder defined to save the results is not a valid path.")
						folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
					}
				} else {
					message("The folder defined to save the results is not a valid path.")
					folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
				}
			} else {
				message("The path to the folder where the results will be saved must be specified as character.")
				folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
			}
		}
		#### cutOff Report ####
		sheet <- output
		store <- file.path(folder, paste(sheet, "_cutOff.csv", sep = ""))
		inputTest <- file.access(store, 0)
		if (inputTest == 0){
			addFile <- 1
			while(inputTest == 0){
				store <- file.path(folder, paste(sheet, addFile, "_cutOff.csv", sep = ""))
				inputTest <- file.access(store, 0)
				addFile <- addFile + 1
			}
		}
		write.csv(smallData, file = store, row.names = FALSE)
		#### Total Report ####
		sheet <- output
		store <- file.path(folder, paste(sheet, "Total.csv", sep = ""))
		inputTest <- file.access(store, 0)
		if (inputTest == 0){
			addFile <- 1
			while(inputTest == 0){
				store <- file.path(folder, paste(sheet, addFile, "Total.csv", sep = ""))
				inputTest <- file.access(store, 0)
				addFile <- addFile + 1
			}
		}
		write.csv(bigData, file = store, row.names = FALSE)
	}
	finalReport <- list(bigData, smallData)
	names(finalReport) <- c("Total", "cutOff")
	return(finalReport)	
}