getGraphs <- function (
		inputData,
		logFile,
		folder, 
		showGraphs = TRUE,
		saveGraphs = TRUE,
		colors,
		scoreCut = 12) {
		
		if (showGraphs == FALSE && saveGraphs == FALSE){
			stop("The arguments showGraphs and saveGraphs are set to FALSE. No plot will be shown nor saved. Please set one of these arguments to TRUE.")
		}	
	###########################################
	########## Check if CSV with dialog #######
###########################################
isCSVdlg <- function(titleMSG, errorMSG) {
		t = 0
		while (t == 0) {
			checkIfCsv <- dlgOpen(title = titleMSG, multiple = FALSE)$res
			checkIfCsv2 <- basename(checkIfCsv)
			checkIfCsv3 <- unlist(strsplit(checkIfCsv2, "\\."))
			checkIfCsv4 <- checkIfCsv3[length(checkIfCsv3)]
			if (toupper(checkIfCsv4) == "CSV") {
				t = 1
				return(checkIfCsv)
			} else {
				dlgMessage(errorMSG)
			}
		}
	}
	###########################################
	########## Check if CSV ###################
###########################################
isCSV <- function(pathFile, errorMSG) {
		t = 0
		checkIfCsv <- pathFile
		checkIfCsv2 <- basename(checkIfCsv)
		checkIfCsv3 <- unlist(strsplit(checkIfCsv2, "\\."))
		checkIfCsv4 <- checkIfCsv3[length(checkIfCsv3)]
		if (toupper(checkIfCsv4) == "CSV") {
			t = 1
			return(t)
		} else {
			return(t)
		}
	}		
	###########################################
	########## Check the input data ###########
###########################################   
if (missing(inputData)) {
		inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
		totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
		message("Input file loaded...")
	} else {
		if (is.data.frame(inputData) == TRUE) {
			totalReport <- inputData
			message("Data frame loaded...")
		} else {
			if (is.character(inputData)) {
				checkIfCsv <- isCSV(inputData)
				if (checkIfCsv == 1) {
					inputTest <- file.access(inputData, 0)
					if (inputTest == 0) {
						totalReport = read.csv(inputData, colClasses = "character", check.names = FALSE)
					} else {
						dlgMessage("The TotalReport specified is not accessible. Please, choose a valid CSV file to be used as input data.")
						inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
						totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
						message("Input file loaded...")
					}
				} else {
					dlgMessage("The TotalReport specified is not in CSV format. Please, choose a valid CSV file to be used as input data.")
					inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
					totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
					message("Input file loaded...")
				}
			} else {
				dlgMessage("The path to the TotalReport must be specified as character string. Please, choose a valid CSV file to be used as input data.")
				inputData <- isCSVdlg("Select the CSV file containing the TotalReport generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
				totalReport <- read.csv(inputData, colClasses = "character", check.names = FALSE)
				message("Input file loaded...")
			}
		}
	}
	
	#### Check the format of the input data and convert to the right format ####
	if (length(grep("RT", names(totalReport)[2])) > 0 && length(grep("DiffRT", names(totalReport)[3])) > 0){
		message("Converting inputData to the format of data(exampleMatchSpectraInverted)[[1]]...")
		totalReport <- data.frame(t(totalReport), stringsAsFactors = FALSE, check.names = FALSE)
		names(totalReport) <- totalReport[1,]
		totalReport <- totalReport[-1,]
		totalReport <- cbind(Name = as.character(row.names(totalReport)), totalReport, stringsAsFactors = F)
		row.names(totalReport) <- 1:nrow(totalReport)
		message("Done!")
	}
	if (length(grep("RT", row.names(totalReport))) > 0 && length(grep("DiffRT", row.names(totalReport))) > 0){
		totalReport <- cbind(Name = as.character(row.names(totalReport)), totalReport, stringsAsFactors = F)
		row.names(totalReport) <- 1:nrow(totalReport)
	}
	###### Remove intesnities below cutOff #####
	removeInt <- function(data, cutOff){
		scores <- data[seq(3,length(data), 4)]
		intReport <- data[seq(4,length(data), 4)]
		suppressWarnings(intReport[as.numeric(scores) < cutOff] <- NA) 
		data[seq(4,length(data), 4)] <- intReport
		return(data)
	}
	
	totalReport <- data.frame(apply(totalReport, 2, function(x) removeInt(x, scoreCut)), stringsAsFactors = F, check.names = FALSE)
	names(totalReport)[1] <- "Name"
	
		###########################################
	########## Check the log file ###########
########################################### 
	if (missing(logFile)) {
		logFile <- isCSVdlg("Select the CSV file containing the log file generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
		logData <- read.csv(logFile, colClasses = "character", check.names = FALSE)
		message("Input file loaded...")
	} else {
		if (is.data.frame(logFile)) {
			logData <- logFile
			message("Data frame loaded...")
		} else {
			if (is.character(logFile)) {
				checkIfCsv <- isCSV(logFile)
				if (checkIfCsv == 1) {
					inputTest <- file.access(logFile, 0)
					if (inputTest == 0) {
						logData = read.csv(logFile, colClasses = "character", check.names = FALSE)
					} else {
						dlgMessage("The input file specified is not accessible. Please, choose a valid CSV file to be used as input data.")
						logFile <- isCSVdlg("Select the CSV file containing the log file generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
						logData <- read.csv(logFile, colClasses = "character", check.names = FALSE)
						message("Input file loaded...")
					}
				} else {
					dlgMessage("The input file specified is not in CSV format. Please, choose a valid CSV file to be used as input data.")
					logFile <- isCSVdlg("Select the CSV file containing the log file generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
					logData <- read.csv(logFile, colClasses = "character", check.names = FALSE)
					message("Input file loaded...")
				}
			} else {
				dlgMessage("The path to the input data must be specified as character string. Please, choose a valid CSV file to be used as input data.")
				logFile <- isCSVdlg("Select the CSV file containing the log file generated by MetaBox", "The input file MUST be in the format of comma-separated value (csv). Please, choose an input file showing the extension .csv.")
				logData <- read.csv(logFile, colClasses = "character", check.names = FALSE)
				message("Input file loaded...")
			}
		}
	}

			
	################### Check if valid folder ###########################
	validFolder <- function(pathFolder, errorMSG1, errorMSG2, titleMSG1) {
		## Is character? ###
		if (is.character(pathFolder)) {
			isFolder <- file.access(as.character(pathFolder), 0)
			if (isFolder == 0) {
				isFolder <- file.info(pathFolder)
				if (isFolder$isdir == TRUE) {
					pathFolder <- pathFolder
				} else {
					dlgMessage(errorMSG2)
					pathFolder <- dlgDir(title = titleMSG1)$res
				}
			} else {
				dlgMessage(errorMSG2)
				pathFolder <- dlgDir(title = titleMSG1)$res
			}
		} else {
			### If not caharacter ###
			dlgMessage(errorMSG1)
			pathFolder <- dlgDir(title = titleMSG1)$res
		}
		return(pathFolder)
	}
	######################################################################
	title1 <- "Select the folder containing the GC-MS data in CDF format (NetCDF or AIA)."
	error1 <- "The dataFolder is not defined as character. A new window will open allowing you to select a valid folder."
	error2 <- "The dataFolder defined in the log file used is not a valid folder. A new window will open allowing you to select a valid folder containing the CDF files in their respective subfolders."
	
		######################################################
	############# Function to check if file is CDF #######
######################################################
isCDFdlg <- function(titleMSG, errorMSG) {
		t = 0
		while (t == 0) {
			checkIfCDF <- dlgOpen(title = titleMSG, multiple = FALSE)$res
			checkIfCDF2 <- basename(checkIfCDF)
			checkIfCDF3 <- unlist(strsplit(checkIfCDF2, "\\."))
			checkIfCDF4 <- checkIfCDF3[length(checkIfCDF3)]
			if (toupper(checkIfCDF4) == "CDF") {
				t = 1
				return(checkIfCDF)
			} else {
				dlgMessage(errorMSG)
			}
		}
	}
	#######################################################
	################# Check if CDF - No dialog box ########
#######################################################
isCDF <- function(pathFile, errorMSG) {
		t = 0
		checkIfCDF <- pathFile
		checkIfCDF2 <- basename(checkIfCDF)
		checkIfCDF3 <- unlist(strsplit(checkIfCDF2, "\\."))
		checkIfCDF4 <- checkIfCDF3[length(checkIfCDF3)]
		if (toupper(checkIfCDF4) == "CDF") {
			t = 1
			return(t)
		} else {
			return(t)
		}
	}
	
	gcFile <- 0
	
	if (as.character(logData$DataFolder[1]) == "none"){
		message("Single file under analysis...")
		file <- as.character(logData$File[1])
		verifCDF <- isCDF(file, "The specified file is not a CDF file.")
		if (verifCDF == 1) {
			inputTest <- file.access(file, 0)
			if (inputTest == 0) {
				dataGC <- xcmsRaw(file)
			} else {
				message("The CDF file in the logFile is not accessible. Please choose a valid CDF file.")
				file <- isCDFdlg("Select the CDF file to be analyzed.", "The selected file is not a CDF file.")
				dataGC <- xcmsRaw(file)
			}
			gcFile <- 1
		} else {
			if (exists(file)){
				if (class(get(file)) == "xcmsRaw"){
					dataGC <- get(file)
					gcFile <- 2
				} else {
					message("The CDF file in the logFile is not accessible. Please choose a valid CDF file.")
					file <- isCDFdlg("Select the CDF file to be analyzed.", "The selected file is not a CDF file.")
					dataGC <- xcmsRaw(file)
					gcFile <- 1
				}
			} else {
				message("The CDF file in the logFile is not accessible. Please choose a valid CDF file.")
				file <- isCDFdlg("Select the CDF file to be analyzed.", "The selected file is not a CDF file.")
				dataGC <- xcmsRaw(file)
				gcFile <- 1
			}
		}
		findCDF <- data.frame(findCDFpre = file, V2 = "Cond1", SampleName = basename(file), stringsAsFactors = F, check.names = FALSE)
		replicates <- as.character(findCDF[, 2])
		reps <- factor(replicates)
		expConditions <- nlevels(reps)
		
	} else {
		dataFolder <- validFolder(as.character(logData$DataFolder[1]), error1, error2, title1)
		### Search subfolders for defining experimental conditions #####
		expConditions <- file.info(list.files(dataFolder, full.names = TRUE))
		expConditions <- expConditions[expConditions$isdir == TRUE, ]
		### If no subfolders found ###
		if (nrow(expConditions) == 0) {
			dlgMessage("The dataFolder specified contains no subfolders defining experimental conditions. We will look for CDF files...")
			findCDF <- list.files(dataFolder, ".cdf", ignore.case = TRUE, full.names = TRUE)
			findCDFNames <- list.files(dataFolder, ".cdf", ignore.case = TRUE, full.names = FALSE)
			if (length(findCDF) == 0) {
				stop("There is no CDF files in the dataFolder specified.")
			}
			message("CDF files found...")
			expConditions <- as.numeric(dlgList(c(1:1000), title = "How many experimental conditions will be analyzed?")$res)
			findCDF <- data.frame(findCDF)
			for (i in 1:expConditions) {
				samples <- dlgList(findCDFNames, title = paste("Select the samples belonging to condition", i), multiple = TRUE)$res
				findCDF[which(findCDFNames == samples), 2] <- paste("ExpCond_", i, sep = "")
			}
			replicates <- as.character(findCDF[, 2])
			reps <- factor(replicates)
			expConditions <- nlevels(reps)
			getName <- function(data){
				file <- basename(as.character(data))
				file <- gsub(".CDF", "", file, ignore.case = TRUE)
				return(file)
			}
			findCDF$SampleName <- apply(findCDF[1], 1, function(x) getName(x))
		} else {
			findCDF <- c()
			message("Subfolders representing experimental conditions found...")
			subfolders <- data.frame(basename(row.names(expConditions)))
			names(subfolders)[1] <- "ExperimentalConditions"
			pandoc.table(subfolders)
			for (i in 1:nrow(expConditions)) {
				findCDFpre <- list.files(row.names(expConditions[i, ]), ".cdf", ignore.case = TRUE, full.names = TRUE)
				if (length(findCDFpre) == 0) {
					message(paste("The subfolder", basename(row.names(expConditions[i, ])), "contains no CDF files. It will not be considered in this analysis."))
				} else {
					findCDFpre <- data.frame(findCDFpre)
					findCDFpre[2] <- basename(row.names(expConditions[i, ]))
					findCDF <- rbind(findCDF, findCDFpre)
				}
			}
			replicates <- as.character(findCDF[, 2])
			reps <- factor(replicates)
			expConditions <- nlevels(reps)
			getName <- function(data){
				file <- basename(as.character(data))
				file <- gsub(".CDF", "", file, ignore.case = TRUE)
				return(file)
			}
			findCDF$SampleName <- apply(findCDF[1], 1, function(x) getName(x))
		}
	}
		if (length(findCDF) == 0){
			stop("The subfolders of the folder specified to matchSpectra contain no CDF files. Please organize the CDF files in individual subfolders representing each experimental condition and alocate this set of folders inside of a main folder. The function matchSpectra will then inspect the main folder for subfolders containing CDF files. See ?matchSpectra for more details.")
		}
	
	######### Check if a CSV or .msl file ############
	isCSVORmsldlg <- function(titleMSG, errorMSG) {
		t = 0
		while (t == 0) {
			checkIfCsv <- dlgOpen(title = titleMSG, multiple = FALSE)$res
			checkIfCsv2 <- basename(checkIfCsv)
			checkIfCsv3 <- unlist(strsplit(checkIfCsv2, "\\."))
			checkIfCsv4 <- checkIfCsv3[length(checkIfCsv3)]
			if (toupper(checkIfCsv4) == "CSV") {
				t = 1
				return(c(checkIfCsv, "csv"))
			} else {
				if (toupper(checkIfCsv4) == "MSL") {
					t = 1
					return(c(checkIfCsv, "msl"))
				} else {
					dlgMessage(errorMSG)
				}
			}
		}
	}
	
	####################################################
	######### Check if a CSV or .msl file  NO DIALOG BOX ############
isCSVORmsl <- function(pathfile, errorMSG) {
		t = 0
		checkIfCsv2 <- basename(pathfile)
		checkIfCsv3 <- unlist(strsplit(checkIfCsv2, "\\."))
		checkIfCsv4 <- checkIfCsv3[length(checkIfCsv3)]
		if (toupper(checkIfCsv4) == "CSV") {
			t = "csv"
			return(t)
		} else {
			if (toupper(checkIfCsv4) == "MSL") {
				t = "msl"
				return(t)
			} else {
				t = 0
				return(t)
			}
		}
	}
	####################################################  
	##### If no ionLib specified: the user can point to a CSV file or to the .msl file from AMDIS library ####
	message("Loading the ion library specified in the logFile...")
		ionLib <- as.character(logData$Library[1])
		checkIfCsv <- isCSVORmsl(ionLib, "The ionLib specified is no accessible.")
		if (checkIfCsv != 0) {
			inputTest <- file.access(ionLib, 0)
			if (inputTest == 0) {
				if (checkIfCsv == "csv") {
					ionLibFile <- ionLib
					ionLib = read.csv(ionLib, colClasses = "character")
					if (ncol(ionLib) != 9) {
						stop("The ionLib selected seems to have a different number of columns than the ionLib used by MetaBox. See data(ionLib) for an example of ionLib.")
					}
					if (sum(names(ionLib) != c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")) > 0) {
						message("The column names of the ionLib selected does not match the expected names of an ionLib used by MetaBox. We will try to modify the column names. See data(ionLib) for an example of the expected column names.")
						names(ionLib) <- c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")
					}
					message("ionLib loaded...")
				} else {
					if (checkIfCsv == "msl") {
						ionLibFile <- ionLib
						message("Converting AMDIS library...")
						ionLib <- buildLib(ionLib, save = FALSE, verbose = FALSE)
						message("ionLib built and loaded...")
					}
				}
			} else {
				dlgMessage("The ionLib specified is not accessible. Please, choose the CSV file or msl file containing the mass spectral library that was used by MetaBox to generate the totalReport.")
				ionLib <- isCSVORmsldlg("Select a CSV file containing the ionLib or the .msl file of the AMDIS library in use.", "The selected file is not a CSV nor a msl file")
				if (ionLib[2] == "csv") {
					ionLibFile <- ionLib[1]
					ionLib = read.csv(ionLib[1], colClass = "character")
					if (ncol(ionLib) != 9) {
						stop("The ionLib selected seems to have a different number of columns than the ionLib used by MetaBox. See data(ionLib) for an example of ionLib.")
					}
					if (sum(names(ionLib) != c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")) > 0) {
						message("The column names of the ionLib selected does not match the expected names of an ionLib used by MetaBox. We will try to modify the column names. See data(ionLib) for an example of the expected column names.")
						names(ionLib) <- c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")
					}
					message("ionLib file loaded...")
				} else {
					if (ionLib[2] == "msl") {
						ionLibFile <- ionLib[1]
						message("Converting AMDIS library...")
						ionLib <- buildLib(ionLib[1], save = FALSE, verbose = FALSE)
						message("ionLib built and loaded...")
					}
				}
			}
		} else {
			if (exists(ionLib)){
				ionLib <- get(ionLib)
				if (ncol(ionLib) != 9) {
						stop("The ionLib selected seems to have a different number of columns than the ionLib used by MetaBox. See data(ionLib) for an example of ionLib.")
					}
				if (sum(names(ionLib) != c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")) > 0) {
					message("The column names of the ionLib selected does not match the expected names of an ionLib used by MetaBox. We will try to modify the column names. See data(ionLib) for an example of the expected column names.")
					names(ionLib) <- c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")
				}
				message("ionLib file loaded...")
			} else {
				dlgMessage("The ionLib specified is not accessible. Please, choose the CSV file or msl file containing the mass spectral library that was used by MetaBox to generate the totalReport.")
				ionLib <- isCSVORmsldlg("Select a CSV file containing the ionLib or the .msl file of the AMDIS library in use.", "The selected file is not a CSV nor a msl file")
				if (ionLib[2] == "csv") {
					ionLibFile <- ionLib[1]
					ionLib = read.csv(ionLib[1], colClass = "character")
					if (ncol(ionLib) != 9) {
						stop("The ionLib selected seems to have a different number of columns than the ionLib used by MetaBox. See data(ionLib) for an example of ionLib.")
					}
					if (sum(names(ionLib) != c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")) > 0) {
						message("The column names of the ionLib selected does not match the expected names of an ionLib used by MetaBox. We will try to modify the column names. See data(ionLib) for an example of the expected column names.")
						names(ionLib) <- c("Compound", "ERT", "M1", "M2", "M3", "M4", "R2", "R3", "R4")
					}
					message("ionLib file loaded...")
				} else {
					if (ionLib[2] == "msl") {
						ionLibFile <- ionLib[1]
						message("Converting AMDIS library...")
						ionLib <- buildLib(ionLib[1], save = FALSE, verbose = FALSE)
						message("ionLib built and loaded...")
					}
				}
			}
		}
	ionLib[2] <- as.numeric(as.character(ionLib[, 2]))
	##################################################################
	############### Set folder if SAVE == TRUE #######################
	##################################################################
	if (saveGraphs) {
		if (missing(folder)) {
			folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
		} else {
			if (is.character(folder)) {
				isFolder <- file.access(as.character(folder), 0)
				if (isFolder == 0) {
					isFolder <- file.info(folder)
					if (!isFolder$isdir) {
						message("The folder defined to save the results is not a valid path.")
						folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
					}
				} else {
					message("The folder defined to save the results is not a valid path.")
					folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
				}
			} else {
				message("The path to the folder where the results will be saved must be specified as character.")
				folder = dlgDir(title = "Select the folder where the output file will be saved.")$res
			}
		}
	}
	####### Get samples from totalReport ######
	numberOfSamples <- length(row.names(totalReport))/4
	sampleIndex <- seq(4, length(row.names(totalReport)), 4)		
	sampleNames <- as.character(totalReport[sampleIndex, 1])
	
	####### Load the GC-MS data ####
	findCDF <- findCDF[which(findCDF$SampleName %in% sampleNames),]	
	if (gcFile == 2){
		gcData <- list(dataGC)
	} else {
		gcData <- apply(findCDF[1], 1, function(x) xcmsRaw(x))
	}
	names(gcData) <- findCDF$SampleName
	conditions <- findCDF$V2[!duplicated(findCDF$V2)]
	###### Define searchWindow ######
	searchWindow <- as.numeric(logData$SearchWindow[1])
	#### Define colors ###
	if (missing(colors)){
		colors <- palette()[1:length(conditions)]
	}
	colorsDots <- c()
	for (i in 1:length(conditions)){
		colorsDots <- c(colorsDots, rep(colors[i], sum(findCDF$V2 == conditions[i])))
	} 
	
	
	###### Function to get IMFs intensities ######
	getIMFInt <- function(CDFdata, libRow, searchWindow) {
		imfs <- as.numeric(libRow[1,3:6])
		RTTarget <- c(as.numeric(libRow[1, 2]) - searchWindow, as.numeric(libRow[1, 2]) + searchWindow) 	
		ionFinalT <- lapply(as.list(imfs), function(x) extract_ion(CDFdata, as.numeric(x), RTTarget))	
		ionFinalTframe <- as.data.frame(ionFinalT, stringsAsFactors = FALSE)
		ionFinalTframe <- ionFinalTframe[c(1,2,4,6,8)]
		Tic <- which(round(CDFdata@scantime) %in% round(ionFinalTframe[, 1] * 60))
		ionFinalTframe$Tic <- CDFdata@tic[Tic]
		return(ionFinalTframe)
	}
	plotIMFs <- function(imfInt, conditions, RTMeanInt, meanInt, colors){
		try(lines(imfInt[, 1], imfInt[, 6], type = "l", col = "grey"), TRUE)
		try(lines(imfInt[, 1], imfInt[, 2], type = "l", col = "firebrick3"), TRUE)
		try(lines(imfInt[, 1], imfInt[, 3], type = "l", col = "dodgerblue4"), TRUE)
		try(lines(imfInt[, 1], imfInt[, 4], type = "l", col = "forestgreen"), TRUE)
		try(lines(imfInt[, 1], imfInt[, 5], type = "l", col = "goldenrod3"), TRUE)
		for (i in 1:length(conditions)){
			try(lines(RTMeanInt, as.numeric(meanInt[i,]), col = colors[i], lty = 3, cex = 1.3), TRUE)
		}
	}
	
	if (showGraphs | saveGraphs){
		initPar <- par()
		dev.off()
	}
	
	plotChrom <- function(datatotalReport, compoundName, ionLib, gcData, conditions, colors, colorsDots, showGraphs, saveGraphs, findCDF){
		libRow <- ionLib[which(ionLib[,1] == compoundName),]
		if (nrow(libRow) == 1){
		RT <- datatotalReport[seq(1, length(datatotalReport), 4)]
		Intensities <- datatotalReport[seq(4, length(datatotalReport), 4)]
		IMFInt <- lapply(gcData, function(x) getIMFInt(x, libRow, searchWindow))
		maxInt <- max(unlist(lapply(IMFInt, function(x) max(x$Tic, na.rm = TRUE))), na.rm = TRUE)
		rtRange <- c(min(unlist(lapply(IMFInt, function(x) min(x$rt, na.rm = TRUE))), na.rm = TRUE), max(unlist(lapply(IMFInt, function(x) max(x$rt, na.rm = TRUE))), na.rm = TRUE))
		meanInt <- as.data.frame(IMFInt, stringsAsFactors = FALSE)
		RTMeanInt <- meanInt[,1]
		meanInt <- meanInt[seq(2, ncol(meanInt), 6)]
		factorForMean <- factor(findCDF[,2])
		meanInt <- data.frame(apply(meanInt, 1, function(x) tapply(x, factorForMean, mean, na.rm = TRUE)), stringsAsFactors = F)
		if (gcFile == 0){
			names(meanInt) <- RTMeanInt
		} else {
			meanInt <- data.frame(t(meanInt), stringsAsFactors = F)
			names(meanInt) <- RTMeanInt
			row.names(meanInt) <- "Cond1"
		}
		### Plot graph ###
		if (showGraphs == FALSE && saveGraphs){
			folderPath <- file.path(folder, "MetaBoxFigures")
			isFolder <- file.access(as.character(folderPath), 0)
			if (isFolder == 0) {
				isFolder <- file.info(folderPath)
				if (isFolder$isdir) {
					#do nothing
				} else {
					dir.create(folderPath)
				}
			} else {
				dir.create(folderPath)
			}
			name.file <- paste(compoundName, ".pdf", sep = "")
			graphNameChrom <- file.path(folderPath, name.file)
			pdf(graphNameChrom)
			suppressWarnings(par(initPar))
			try(plot(rtRange, c(0,maxInt), main = libRow[1, 1], ylim = c(-50, maxInt), cex.main = 1, bty = "n", axes = T, xlab = "RT(min)", ylab = "Intensity", col = "white"), TRUE)
			###### Function to plot #####
			lapply(IMFInt, function(x) plotIMFs(x, conditions, RTMeanInt, meanInt, colors))
			try(legend("topleft", legend = c("TIC", libRow[1,3:6]), cex = 0.5, col = c("grey", "firebrick3", "dodgerblue4", "forestgreen", "goldenrod3"), lty = c(1, 1, 1, 1, 1), title = "Fragments", lwd = 3, merge = FALSE, bty = "n"), TRUE)
			try(points(RT, Intensities, pch = 19, col = colorsDots), TRUE)
			try(legend("topright", legend = c(conditions, paste("Mean_", conditions, sep = "")), cex = 0.5, col = rep(colors,2), pch = c(rep(19, length(conditions)), rep(NA, length(conditions))), lty = c(rep(NA, length(conditions)), rep(3, length(conditions))), title = "Data classes", merge = FALSE, bty = "n"), TRUE)
			dev.off()
		}
		if (showGraphs){
			suppressWarnings(par(initPar))
			try(plot(rtRange, c(0,maxInt), main = libRow[1, 1], ylim = c(-50, maxInt), cex.main = 1, bty = "n", axes = T, xlab = "RT(min)", ylab = "Intensity", col = "white"), TRUE)
			###### Function to plot #####
			lapply(IMFInt, function(x) plotIMFs(x, conditions, RTMeanInt, meanInt, colors))
			try(legend("topleft", legend = c("TIC", libRow[1,3:6]), cex = 0.5, col = c("grey", "firebrick3", "dodgerblue4", "forestgreen", "goldenrod3"), lty = c(1, 1, 1, 1, 1), title = "Fragments", lwd = 3, merge = FALSE, bty = "n"), TRUE)
			try(points(RT, Intensities, pch = 19, col = colorsDots), TRUE)
			try(legend("topright", legend = c(conditions, paste("Mean_", conditions, sep = "")), cex = 0.5, col = rep(colors,2), pch = c(rep(19, length(conditions)), rep(NA, length(conditions))), lty = c(rep(NA, length(conditions)), rep(3, length(conditions))), title = "Data classes", merge = FALSE, bty = "n"), TRUE)
			if (saveGraphs){
				folderPath <- file.path(folder, "MetaBoxFigures")
				isFolder <- file.access(as.character(folderPath), 0)
				if (isFolder == 0) {
					isFolder <- file.info(folderPath)
					if (isFolder$isdir) {
						#do nothing
					} else {
						dir.create(folderPath)
					}
				} else {
					dir.create(folderPath)
				}
				name.file <- paste(compoundName, ".pdf", sep = "")
				graphNameChrom <- file.path(folderPath, name.file)
				dev.copy2pdf(file = graphNameChrom)
			}
		}		
		counter <<- counter + 1
		return(list(meanInt, IMFInt))
		} else {
		counter <<- counter + 1
		return(list("Not found in ionLib", "Not found in ionLib"))
		}
	}
	compoundNames <- names(totalReport)[2:ncol(totalReport)]
	counter <- 1
	
	finalPloting <- apply(totalReport[-c(1)], 2, function(x) plotChrom(x, compoundNames[counter], ionLib, gcData, conditions, colors, colorsDots, showGraphs, saveGraphs, findCDF))
	return(finalPloting)	
}