\name{buildLib}
\alias{buildLib}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
The function buildLib converts an AMDIS' library into a data frame and/or a CSV file in the format required by MetaBox.
}
\description{
A mass spectral library built using AMDIS is composed of two files: one file with the extension .MSL and another file with extension .CID. The function buildLib converts the .MSL file of an AMDIS' library into a data frame and/or a CSV file in the format required by MetaBox.
}
\usage{
buildLib(AmdisLib, folder, save = TRUE, output = "ion_lib", verbose = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{AmdisLib}{
when AmdisLib is missing, a dialog box will pop up allowing the user to click-and-point to the .MSL file from which the data is to be read. Alternatively, AmdisLib can take a character string naming the path to the .MSL file to be read or the name of an R object (data frame) containing the .MSL file (e.g. data(exampleMSLfile)).
}
  \item{folder}{
when save = TRUE and folder is missing, a pop up dialog box will be presented to the user. The user can then select the directory to which the results will be saved. Alternatively, the argument folder can take a character string naming the path to the folder where the results must be saved.
}
  \item{save}{
a logical vector (TRUE or FALSE) defining if the results must be saved into a CSV file (default = TRUE).
}
  \item{output}{
a character string indicating the name of the CSV file where the results will be stored (default = ion_lib.csv). If a file with the same name exists in the folder specified through the argument folder, the new file will be generated using the name defined in output followed by a number. This number indicates the version of the file, with the most recent version showing the highest number. For example, if the value of output is defined as "results" and a file called "results.csv" already exists in the folder specified in the argument folder, the new CSV file containing the results will be called as "results1.csv".
}
  \item{verbose}{
logical vector indicating if a progress bar should be present while the AMDIS library is converted.
}
}
\details{
The Automated Mass Spectral Deconvolution and Identification System (AMDIS) is a software developed by NIST (http://chemdata.nist.gov/mass-spc/amdis/). It makes use of a mass spectral library composed of two files, a .CID and a .MSL file. The function buildLib allows a quick conversion of the AMDIS' library into a .CSV file with the format required by MetaBox. For this, buildLib requires only the .MSL file of the AMDIS' library in use.
}
\value{
buildLib returns a data frame containing the names of compounds in the first column, their expected retention time (ERT) in the second column, the 4 most abundant ion mass fragments (M1, M2, M3 and M4) in the following 4 columns (R2, R3 and R4) and the expected intensity ratios of ions M2, M3 and M4 in relation to M1.

In addition, buildLib verifies the existence of metabolites expected at similar ERT (less than 1 minute difference) and that use the same ion mass fragment M1. These metabolites are likely to show strong coelution, which may dificult their correct identification. Metabolites showing these characteristics are presented to the user at the end of the library conversion. For these metabolites, we strongly suggest selecting different ion mass fragments in order to avoid coelution effects.  

}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 
}
\author{
Raphael Aggio
}
\note{
The .MSL file of the AMDIS' library used MUST contain the expected RT of each compound, as it is required by MetaBox for compound identification.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{extract_ion}}, \code{\link{filterCDF}}, \code{\link{filterCDFBatch}},\code{\link{filterTotalReport}}, \code{\link{findNoise}}, \code{\link{getGraphs}}, \code{\link{matchSpectra}}, \code{\link{matchSpectraFile}}, \code{\link{plotIons}} 
}
\examples{
	### Load example MSL file #######
	data(exampleMSLfile)
	########### Run BuildLib ########
	CSVLib <- buildLib(exampleMSLfile, save = FALSE, verbose = FALSE)
	########### Print Results #######
	print(CSVLib)	
}