\name{extract_ion}
\alias{extract_ion}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Extract the intensities of a specific ion mass fragment (IMF) during a specific retention time (RT) window.
}
\description{
The function extract_ion extracts the intensities of a specific ion mass fragment (IMF) defined through teh argument "mz" during a specific RT window defined through the argument timeWindow.
}
\usage{
extract_ion(data, mz = 43, timeWindow = c(10.2, 11), by = 0.01)
}

\arguments{
  \item{data}{
a vector containing an xcmsRaw object. See ?xcms::xcmsRaw.
}
  \item{mz}{
a numerical vector indicating the mass-to-charge ratio of the IMF to be extracted.
}
  \item{timeWindow}{
a numerical vector of size 2 containing the initial and the final RT (in minutes) defining the time window to be analysed.
}
  \item{by}{
a numerical vector indicating the resolution of the timeWindow. The value of by will be passed to the function seq() when defining the timeWindow range.
}
}

\value{
The function extract_ion returns a data frame containing two columns: the RT window analysed in the column 1; and the intensities of the specified IMF at each RT analysed in the column 2.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 
}
\author{
Raphael Aggio
}

\seealso{
\code{\link{buildLib}}, \code{\link{filterCDF}}, \code{\link{filterCDFBatch}},\code{\link{filterTotalReport}}, \code{\link{findNoise}}, \code{\link{getGraphs}}, \code{\link{matchSpectra}}, \code{\link{matchSpectraFile}}, \code{\link{plotIons}} 
}
\examples{
	###### Load example CDF file ####################
	CDFfile <- xcmsRaw(unzip(system.file("extdata/130513_REF_SOL2_2_50_50_1.CDF.zip", package = "MetaBox")))
	###### Run exrtact_ion on example CDF file ######
	IntensityOfMZ43 <- extract_ion(CDFfile, mz = 43, timeWindow = c(10.2,11))
	###### Print results ############################
	print(IntensityOfMZ43)
	plot(IntensityOfMZ43, main = "The intensity of IMF 43 from RT 10.2 to 11 min", xlab = "RT", 
		ylab = "Intensity", bty = "n", type = "b", col = "red")
}
