\name{filterCDF}
\alias{filterCDF}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
The function filterCDF removes the background and the abundances of specific ion mass fragments (IMF) from CDF files analyzed by GC-MS. 
}
\description{
filterCDF searches the CDF file for a region of the chromatogram that could be used as reference for removing the background noise and allows removing from this CDF file specific ion mass fragments.  
}
\usage{
filterCDF(data, filterNoise = TRUE, mzListToFilter = c(1:30), save = TRUE, folder, output = "_noNoise", limitScan, plotGraph = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
when data is missing, a dialog box will pop up allowing the user to click-and-point to the CDF file from which the data is to be read. Alternatively, data can take a xcmsRaw object (See ?xcms::xcmsRaw) or a character string naming the path to the CDF file to be read.
}
  \item{filterNoise}{
a logical vector (TRUE or FALSE) indicating wether the background noise should be filtered (default = TRUE) (See details).
}
  \item{mzListToFilter}{
a numeric vector indicating the ion mass fragments to be removed.
}
  \item{save}{
a logical vector (TRUE or FALSE) indicating wether the filtered data should be saved to a CDF file (default = TRUE).
}
  \item{folder}{
if save = TRUE and folder is missing, a new dialog box will pop up allowing the user to click-and-point to the desired folder where the results will be saved. Alternatively, folder may receive a character vector indicating the path to the folder where the new CDF file must be saved.
}
  \item{output}{
when save = TRUE, the value of output (a character vector) will be added to the end of the original CDF file name before saving. For example, if output = "noNoise" (default) and the file "test.CDF" is under analysis, the new CDF files produced by filterCDF will be named as test_noNoise.CDF.
}
  \item{limitScan}{
a numerical value indicating the maximun scan value to be analyzed, if filterNoise = TRUE (See details).
}
  \item{plotGraph}{
a logical value indicating wether results should be plotted (default = TRUE).
}
}
\details{
The background noise is removed in three steps. First, we search for a flat region of the chromatogram showing the lowest abundance in the total ion count. filterCDF uses the strategy proposed by Stein (1999) for finding a flat are of the chromatogram. It consists in analyzing 13 scans at a time and searching for regions where scans are similar to the mean intensity of the selected scans. Then, we collect the intensity values of all the ion mass fragments located at a specific scan in this flat region of the chromatogram. Finally, we subtract the intensity of all the collected ion mass fragments from every scan in the CDF file under analysis.  	
The argument limitScan is used to restrict the regions of the chromatogram that will be searched for a flat region. For example, if limitScan = 1500, a flat region in the chromatogram will be searched only between scans 1 and 1500.
The function write.cdf is used to generate the new CDF file. See ?write.cdf for more details.

}
\value{
filterCDF generates a new xcmsRaw object and/or a CDF file containing the filtered data.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2.

Stein, S. E. (1999). An integrated method for spectrum extraction and compound identification from gas chromatography/mass spectrometry data. Journal of the American Society for Mass Spectrometry, 10(8), 770-781. doi: 10.1016/S1044-0305(99)00047-1.
}
\author{
Raphael Aggio
}
\seealso{
\code{\link{buildLib}}, \code{\link{extract_ion}}, \code{\link{filterCDFBatch}},\code{\link{filterTotalReport}}, \code{\link{findNoise}}, \code{\link{getGraphs}}, \code{\link{matchSpectra}}, \code{\link{matchSpectraFile}}, \code{\link{plotIons}} 
}
\examples{
	########## Load exampleCDFfile ###################
	CDFfile <- unzip(system.file("extdata/130513_REF_SOL2_2_50_50_1.CDF.zip", package = "MetaBox"))
	########## Apply filter CDF to remove ion mas fragments 40, 45 and 48 to 80 ######################
	filteredData <- filterCDF(CDFfile, filterNoise = TRUE, mzListToFilter = c(40,45,48:80), save = FALSE, plotGraph = TRUE)                                                                                                                                                                                                                  
}