\name{filterCDFBatch}
\alias{filterCDFBatch}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
The function filterCDFBatch applies the function filterCDF for all the CDF files contained in a defined folder. 
}
\description{
For all CDF files contained in a specified folder or directory, filterCDFBatch searches for a region of the chromatogram that could be used as reference for background noise removal and allows removing the intensity contribution of specific ion mass fragments.  
}
\usage{
filterCDFBatch(dataFolder, filterNoise = TRUE, mzListToFilter = c(1:30), output = "_noNoise", limitScan, plotGraph = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dataFolder}{
a character vector indicating the path to the main folder or directory containing the CDF files to be filtered. If dataFolder is missing, a new dialog box will pop up allowing the user to point and click at the folder or directory to be used (See details).
}
  \item{filterNoise}{
a logical vector (TRUE or FALSE) indicating wether the background noise should be removed (default = TRUE) (See details).
}
  \item{mzListToFilter}{
a numeric vector indicating the ion mass fragments to be removed.
}
  \item{output}{
the value of the argument output (a character vector) will be added to the end of the name of the original CDF file before saving. For example, if output = "_noNoise" (default) and the file "test.CDF" is under analysis, the new CDF files produced by filterCDFBatch will be named as test_noNoise.CDF.
}
  \item{limitScan}{
a numerical value indicating the maximun scan value to be analyzed, if filterNoise = TRUE (See \code{\link{filterCDF}}).
}
  \item{plotGraph}{
a logical value indicating wether results should be plotted (default = TRUE).
}
}
\details{
filterCDFBatch was developed to automatically detect CDF files belonging to different experimental conditions. Thus, the CDF files analysed by filterCDFBatch must be organised as follows:

	Main_Directory/
		Experimental_Condition_1/
			CDF_File_1_1.CDF
			CDF_File_1_2.CDF
			CDF_File_1_3.CDF
		Experimental_Condition_2/
			CDF_File_2_1.CDF
			CDF_File_2_1.CDF
			CDF_File_2_1.CDF
		....
		
Considering the CDF files organised as above, filterCDFBatch is applied to the main folder "Main_Directory" and it will generate a new directory for each experimental condition in order to store the new filtered CDF files. The result of filterCDFBatch would then be:

	Main_Directory/
		Experimental_Condition_1/
			CDF_File_1_1.CDF
			CDF_File_1_2.CDF
			CDF_File_1_3.CDF
		Experimental_Condition_2/
			CDF_File_2_1.CDF
			CDF_File_2_1.CDF
			CDF_File_2_1.CDF
		Experimental_Condition_1_noNoise/
			CDF_File_1_1_noNoise.CDF
			CDF_File_1_2_noNoise.CDF
			CDF_File_1_3_noNoise.CDF
		Experimental_Condition_2_noNoise/
			CDF_File_2_1_noNoise.CDF
			CDF_File_2_1_noNoise.CDF
			CDF_File_2_1_noNoise.CDF

If limitScan = 3000, a noise region of the chromatogram will be searched only between scans 1 and 3000.
The function write.cdf is used to generate the new CDF file. See ?xcms::write.cdf for more details.
See \code{\link{filterCDF}} for more details about how filterCDFBatch works.

}
\value{
The function filterCDFBatch generates filtered CDF files in subfolders created inside of the directory specified through the argument dataFolder.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 
}
\author{
Raphael Aggio
}
\seealso{
\code{\link{buildLib}}, \code{\link{extract_ion}}, \code{\link{filterCDF}},\code{\link{filterTotalReport}}, \code{\link{findNoise}}, \code{\link{matchSpectra}}, \code{\link{matchSpectraFile}}, \code{\link{plotIons}} 
}
\examples{
	########## Load exampleCDFfile ###################
	CDFfile <- unzip(system.file("extdata/130513_REF_SOL2_2_50_50_1.CDF.zip", package = "MetaBox"))
	########## Apply filter CDF to remove ion mas fragments 40, 45 and 48 to 80 ######################
	filteredData <- filterCDF(CDFfile, filterNoise = TRUE, mzListToFilter = c(40,45,48:80), save = FALSE, plotGraph = TRUE)                                                                                                                                                                                                                   
}