\name{filterTotalReport}
\alias{filterTotalReport}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Filter the Total report generated by the functions \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}.
}
\description{
filterTotalReport is used to filter the Total report generated by the functions \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. This process is performed by removing from each sample those metabolites that received from \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}} a score lower that the value defined in the argument scoreCut.
}
\usage{
filterTotalReport(inputData, scoreCut = 12, save = TRUE, folder, output = "filteredReport")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{inputData}{
when inputData is missing, a dialog box will pop up allowing the user to click-and-point to the CSV file Total.csv generated by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. Alternatively, inputData can take a character string naming the path to the CSV file to be read or the name of an object (data frame) containing the Total report produced by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}.    
}
  \item{scoreCut}{
a numerical vector indicating the minimum score associated to a TRUE metabolite. Metabolites showing a lower score than the value specified through scoreCut will not be part of the results.
}
  \item{save}{
a logical vector (TRUE or FALSE) defining if the results must be saved to a CSV file.
}
  \item{folder}{
when save = TRUE and folder is missing, a pop up dialog box will be presented to the user. The user can then select the directory to which the results will be saved. Alternatively, folder can take a character string naming the path to the folder where the results must be saved.
}
  \item{output}{
a character string indicating the name of the file containing the results.
}
}
\details{ 
The function filterTotalReport is used to filter the Total report generated by the functions \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. It generates two reports containing only metabolites that received a score higher or equal to the value of the argument scoreCut. For example, if scoreCut = 15, filterTotalReport will filter the report generated by matchSpectra and generate reports containing only metabolites that received a score of 15 or higher when analyzed by the functions \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. 

}
\value{
filterTotalReport returns a list containing 2 data frames: Total and cutOff. When save = TRUE, these two data frames are saved to two csv files.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 
}
\author{
Raphael Aggio
}
\seealso{
\code{\link{buildLib}}, \code{\link{extract_ion}}, \code{\link{filterCDF}}, \code{\link{filterCDFBatch}},\code{\link{findNoise}}, \code{\link{matchSpectra}}, \code{\link{matchSpectraFile}}, \code{\link{plotIons}} 
}
\examples{
	########### Load data ##########
	data(exampleMatchSpectraInverted)
	##### Run filterTotalReport ####
	dataTest <- exampleMatchSpectraInverted$cutOff
	##### Plot metabolite abundances ######
	replicates <- factor(dataTest$Replicates)
	dataTest <- apply(dataTest[-1], 2, function(x) tapply(as.numeric(x), replicates, mean, na.rm = TRUE))
	par(mar = c(8,5,6,2), mfrow = c(1,2))
	barplot(dataTest, las = 2, beside = TRUE, cex.names = 0.8, cex.axis = 0.8, 
		main = "Compound abundances before filtering", ylab = "Intensities", col = c("black", "red"))
	legend("topleft", legend = c("100uL", "50uL"), pch = 19, col = c("black", "red"), box.lwd = 0)
	######### Aplly filter selecting compounds that received scores higher or equal to 16 ######
	testFilter <- filterTotalReport(exampleMatchSpectraInverted$Total, scoreCut = 16, save = FALSE)
	######### Plot results ########
	testFilter <- testFilter$cutOff
	replicates <- factor(testFilter$Replicates)
	dataTest <- apply(testFilter[-c(1,2)], 2, function(x) tapply(as.numeric(x), replicates, mean, na.rm = TRUE))
	barplot(dataTest, las = 2, beside = TRUE, cex.names = 0.8, cex.axis = 0.8, 
		main = "Compound abundances after filtering", ylab = "Intensities", col = c("black", "red"))
	legend("topleft", legend = c("100uL", "50uL"), pch = 19, col = c("black", "red"), box.lwd = 0)
}