\name{findNoise}
\alias{findNoise}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Find a scan of a xcmsRaw object that represents the background noise associated to a file.
}
\description{
findNoise searches for flat regions of a chromatogram to be used as a reference for the background noise associated.
}
\usage{
findNoise(data, folder, name.file = "findNoiseGraph", saveGraph = FALSE, limitScan, plotGraph = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
a xcmsRaw object (See ?xcmsRaw).
}
  \item{folder}{
a character string naming the path to the folder where the plot produced by findNoise should be saved, if saveGraphs = TRUE.
}
  \item{name.file}{
the name of the file that should store the graph generated by findNoise when saveGraphs = TRUE.
}
  \item{saveGraph}{
a logical vector (TRUE or FALSE) indicating if a graph must be produced by findNoise. If saveGraphs = TRUE, findNoise produces a graph showing the RT used as noise reference.
}
  \item{limitScan}{
a numerical vector indicating the maximun scan to be analyzed. It can be used to avoid using scans located at the end of the chromatogram, for example. 
}
  \item{plotGraph}{
a logical vector indicating if the results should be plotted. 
}
}
\details{
If limitScan = 3000, a noise region of the chromatogram will be searched only between scans 1 and 3000.

findNoise uses the strategy proposed by Stein (1999) for finding a flat are of the chromatogram. It consists in analyzing 13 scans at a time and searching for regions where scans are similar to the mean intensity of the selected scans.
}
\value{
findNoise generates a data frame containing the ion mass fragments and their respective intensities in the scan chosen to represent the background noise. If no noise region is found it returns a vector of length 0.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 

Stein, S. E. (1999). An integrated method for spectrum extraction and compound identification from gas chromatography/mass spectrometry data. Journal of the American Society for Mass Spectrometry, 10(8), 770-781. doi: 10.1016/S1044-0305(99)00047-1.

}
\author{
Raphael Aggio
}
\seealso{
\code{\link{buildLib}}, \code{\link{extract_ion}}, \code{\link{filterCDF}}, \code{\link{filterCDFBatch}},\code{\link{filterTotalReport}}, \code{\link{getGraphs}}, \code{\link{matchSpectra}}, \code{\link{matchSpectraFile}}, \code{\link{plotIons}}
}
\examples{
	##### Load example CDF ####
	CDFfile <- xcmsRaw(unzip(system.file("extdata/130513_REF_SOL2_2_50_50_1.CDF.zip", package = "MetaBox")))
	##### Run findNoise ########
	Noise <- findNoise(CDFfile, saveGraph = FALSE, plotGraph = FALSE, limitScan = 1000)
	##### Print resuls #########
	barplot(Noise[,2], names.arg = round(Noise[,1], 1), las = 2, width = 1, beside = TRUE, 
		cex.names = 0.6, cex.axis = 0.7, xlab = "m/z", ylab = "Raw Intensity", 
		main = "The raw intensities of ion mass fragments found at the flat region", col = "red")                                                                                                                                                                                                                  
}