\name{getGraphs}
\alias{getGraphs}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
getGraphs produces chromatograms showing the retention time position and intensity associated to each compound reported by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. 
}
\description{
getGraphs is used to generate chromatograms showing the exact position where compounds were identified by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. In addition, getGraphs returns the average intensity of each sampling point in samples from each experimental condition. 
}
\usage{
getGraphs(inputData, logFile, folder, showGraphs = TRUE, saveGraphs = TRUE, colors, scoreCut = 12)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{inputData}{
when inputData is missing, a dialog box will pop up allowing the user to click-and-point to the CSV file Total.csv generated by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. Alternatively, inputData can take a character string naming the path to the CSV file to be read or the name of an object (data frame) containing the Total report produced by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}.    
}
  \item{logFile}{
when logFile is missing, a dialog box will pop up allowing the user to click-and-point to the CSV file logFile.csv generated by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. Alternatively, inputData can take a character string naming the path to the CSV file to be read or the name of an object (data frame) containing the log file produced by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}.    
}
  \item{folder}{
when saveGraphs = TRUE and folder is missing, a pop up dialog box will be presented to the user. The user can then select the directory to which the results will be saved. Alternatively, folder can take a character string naming the path to the folder where the results must be saved.
}
  \item{showGraphs}{
a logical vector (TRUE or FALSE) defining if the chromatogram of each compound should be plotted.
}
  \item{saveGraphs}{
a logical vector (TRUE or FALSE) defining if the chromatogram of each compound should be stored in a pdf file.
}
  \item{colors}{
the color used to represent each experimental condition.
}
  \item{scoreCut}{
a numerical vector defining the minimun accepted score returned by the functions \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. Metabolites showing a score lower than the scoreCut will not be plotted.
}
}
\details{ 
The function getGraphs is used to plot the chromatograms of compounds reported in the Total report generated by the functions \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}. It uses the logFile produced by the functions \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}} in order to collect the parameters used in the analysis, such as the folder containing the GC-MS files, the searchWindow used and the ion library used. 

}
\value{
getGraphs generates one chromatogram for each compound. In this chromatogram it is highlighet the positions where the compound was identified in the different samples analysed. In addition, it returns a list containing the average intensity of each metabolite for each experimental condition and the intensity of each ion mass fragment in each analysed sample during the searchWindow applied by \code{\link{matchSpectra}} or \code{\link{matchSpectraFile}}.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 
}
\author{
Raphael Aggio
}
\seealso{
\code{\link{buildLib}}, \code{\link{extract_ion}}, \code{\link{filterCDF}}, \code{\link{filterCDFBatch}},\code{\link{findNoise}}, \code{\link{matchSpectra}}, \code{\link{matchSpectraFile}}, \code{\link{plotIons}} 
}
\examples{
	#### Load library and GC-MS data ##
	data(exampleIonLib)
	CDFfile <- unzip(system.file("extdata/130513_REF_SOL2_2_50_50_1.CDF.zip", package = "MetaBox"))
	#### Run matchSpectraFile ####
	dataTest <- matchSpectraFile(CDFfile, exampleIonLib, scoreCut = 12, save = FALSE)
	#### Run getGraphs ###
	testing <- getGraphs(dataTest$Total, dataTest$logFile, showGraphs = TRUE, saveGraphs = FALSE, colors = c("black"), scoreCut = 12)
	print(testing)
}