\name{matchSpectraFile}
\alias{matchSpectraFile}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Identify metabolites in a single CDF file analyzed by GC-MS.
}
\description{
matchSpectraFile uses a mass spectral library to identify metabolites in a single CDF file analyzed by GC-MS.
}
\usage{
matchSpectraFile(file, ionLib, searchWindow = 0.2, matchFactor = 0.7, correlation = 0.95, 
	scoreCut = 10, noiseThreshold = 0, save = TRUE, folder, output = "MatchReport", peakFindMethod = 2, 
	invertOutput = TRUE, showGraphs = TRUE, saveGraphs = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{file}{
when file is missing, a dialog box will pop up allowing the user to click-and-point to a CDF file. Alternatively, file can take a character string naming the path to the CDF file to be analysed or to a vector containing a xcmsRaw object (See ?xcms::xcmsRaw for more details).
}
  \item{ionLib}{
when ionLib is missing, a dialog box will pop up allowing the user to click-and-point to a .CSV file containing the mass spectral library to be used or the .MSL file of an AMDIS library (See Details). Alternatively, ionLib can take a character string naming the path to the .CSV or .MSL file to be used. Additionally, ionLib may take the vector name of a data frame containing the mass spetral library (See ?exampleIonLib for more details).
}
  \item{searchWindow}{
a numerical vector indicating the retention time (RT) window to search each compound present in the mass spectral library used (See Details). 
}
  \item{matchFactor}{
a numerical vector from 0 to 0.99 indicating the minimum level of similarity between the observed and the expected spectrum in order to consider a positive match. The expected spectrum is the spectrum present in the spectral library used, while the observed spectrum is the spectrum obtained from the CDF file under analysis (See Details). 
}
  \item{correlation}{
a numerical vector from 0 to 0.99 indicating the minimum level of correlation (Pearson's correlation) between ion mass fragments originated from the same compound in order to consider a positive correlation. We assume the intensities of ion mass fragments originated from the same compound are likely to have a strong positive correlation during a time window of RT +- 0.07min. 
}
  \item{scoreCut}{
a numerical vector from 0 to 18 indicating the minimum score to consider a true identification. The score defined in scoreCut will be used to generate a compacted report containing only the identified compounds that received a score higher or equal to the value of the scoreCut (See Details). 
}
  \item{noiseThreshold}{
a numerical vector indicating the minimum intensity of the ion mass fragments to be considered in the analysis. For example, ion mass fragments with intensity lower than 1000 will not be analyzed if noiseThreshold = 1000.
}
  \item{save}{
a logical vector (TRUE or FALSE) indicating if the reports produced by matchSpectra should be saved. If save = TRUE, two reports are generated and stored in CSV files: a Total report and a CutOff report (See Value). A log file containing the details of the analysis will also be saved.
}
  \item{folder}{
when save = TRUE and folder is missing, a pop up dialog box will be presented to the user. The user can then select the directory to which the results will be saved. Alternatively, the argument folder can take a character string naming the path to the folder where the results must be saved.
}
  \item{output}{
a character string naming the CSV files produced by matchSpectra. Three CSV files are generated by matchSpectra. One CSV file is named using the value of output + CutOff and stores the metabolites identified with their abundances in the different samples analysed. This report contains only metabolites that received a higher score than the value defined through scoreCut. The other CSV file generated by matchSpectra contains the name of every metabolite analysed with their RT, the difference between expected and observed RT, the score calculated by matchSpectra and their intensity (the intensity of the reference ion or most abundant ion mass fragment). Finally, a log file named as logFile + output will also be saved.    
}
  \item{peakFindMethod}{
the numerical values of 1, 2, or 3. Each numerical value indicates one specific method for finding peaks in a chromatogram (See Details). 
}
  \item{invertOutput}{
a logical vector (TRUE or FALSE) indicating if the results of matchSpectra should be inverted before reporting. When invertOutput = FALSE, the reports produced by matchSpectra are structured with names of compounds in the first column and the intensity of each compound in each sample in the subsequent columns. If invertOutput = TRUE, the reports generated by matchSpectra are structured with the name of samples in the first column and the intensities of compounds in each sample in the subsequent columns (See data(exampleMatchSpectra) and data(exampleMatchSpectraInverted)).  
}
  \item{showGraphs}{
a logical vector (TRUE or FALSE) defining if the graphs produced by matchSpectra should be presented to the user.
}
  \item{saveGraphs}{
a logical vector (TRUE or FALSE) defining if the graphs produced by matchSpectra should be saved. If saveGraphs = TRUE, the graphs produced by matchSpectra are stored in a folder named MetaBoxGraphOutput located inside of the folder defined as the main folder (dataFolder). If the folder MetaBoxGraphOutput does not exist, it will be created. If the folder MetaBoxGraphOutput already exists, existing files will be replaced by the new files.
}
}
\details{

matchSpectra searches the CDF files for each metabolite present in the mass spectral library (ionLib) used. For each compound in ionLib, matchSpectra searches for chromatographic peaks associated to the their four ion mass fragments at the time window defined as the expected retention time (defined in the ionLib) +- the value defined in the argument searchWindow. The argument searchWindow defines the amount of minutes forward and backward of the expected RT for searching a metabolite. For example, if searchWindow = 0.2 and the expected RT of Alanine is 10.2, Alanine will be searched in the RT window 10 min to 10.4 min. Each chromatographic peak found during this RT window is represented by its specific retention time (RT). Each RT found then receives points according to:
	1 - If the four ion mass fragments associated with the metabolite being analysed are found with intensities > 0 (4 points);
	2 - If the four ion mass fragments associated with the metabolite being analysed show their peaks at the same RT (4 points);
	3 - If the four ion mass fragments associated with the metabolite being analysed show the highest peak intensity at the same RT (4 points);
	4 - If the ratios between ion mass fragments 2 and 1, 3 and 1, and 4 and 1 are similar to the respective ratios present in the mass spectral library (ionLib columns R2, R3 and R4). The argument matchFactor defines the similarity between these ratios, where 0 means no similarity required and 0.99 means high similarity required. If matchFactor = 0.7, the acquired mass spectrum must be at least 70 per cent similar to a spectrum present in the ionLib to be considered a TRUE compound (3 points);
	5 - If the ion mass fragments associated with the metabolite being analysed show a correlation (Pearsons correlation) higher than the value defined in the argument correlation (3 points);
	
	Total score = 4 + 4 + 4 + 3 + 3 = 18.
	
If different compounds are found at the same RT, the compound that received the highest score is selected as true.

matchSpectra allows the user to choose between 3 different methods for peak finding. In method 1, a chromatographic peak is detected if the intensity of a GC-MS scan is higher than the two previous scans and higher than the two next scans. In method 2, a chromatographic peak is detected using the first derivative of the chromatogram. In this case, a peak is found every time the signal of the first derivative changes from positive to negative. Finally, the method 3 uses the function getLocalMaximumCWT of the package MassSpecWavelet to find potential peaks. The method 1 can be slower than methods 2 and 3. As different peakFind methods may result in different peaks found, the results of matchSpectra may change according to the peakFind method used.

}
\value{
matchSpectra returns a list containing 4 data frames: Total report, cutOff report, the ionLib used and the log file. If save = TRUE, matchSpectra stores Total report, cutOff report and log file in CSV files. In addition, if saveGraphs = TRUE, matchSpectra generates multiple graphs in pdf format saved in a folder called MetaBoxGraphOutput and located in the main folder defined through the argument dataFolder. See ?exampleMatchSpectra or ?exampleMatchSpectraInverted.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 
}
\author{
Raphael Aggio
}
\seealso{
\code{\link{buildLib}}, \code{\link{extract_ion}}, \code{\link{filterCDF}}, \code{\link{filterCDFBatch}},\code{\link{filterTotalReport}}, \code{\link{matchSpectra}}, \code{\link{findNoise}}, \code{\link{plotIons}} 
}
\examples{
	##### Load ionLib ####
	data(exampleIonLib)
	##### Load Sample ####
	CDFfile <- unzip(system.file("extdata/130513_REF_SOL2_2_50_50_1.CDF.zip", package = "MetaBox"))
	### Analyze sample ###
	exampleTest <- matchSpectraFile(CDFfile, exampleIonLib, scoreCut = 12, save = FALSE, showGraphs = TRUE, saveGraphs = FALSE)
	### Print results ####
	print(exampleTest)
	graphics.off()
	par(mar = c(8,5,4,2))
	barplot(as.numeric(exampleTest$cutOff), names.arg = names(exampleTest$cutOff), las = 2, cex.axis = 0.5, cex.names = 0.8, 
		ylab = "Intensity", main = "Compound abundances", col = "orange")	
}