\name{plotIons}
\alias{plotIons}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plot all the ions detected by the GC-MS during a specific retention time (RT) window.
}
\description{
plotIons calculates the intensity of all the ion mass fragments detected by the GC-MS during a specific RT window. It plots these ions in a chromatogram and store them in a data frame or CSV file. plotIons also allows the user to interactivelly select a region of the chromatogram to identify its respective mass spectrum. 
}
\usage{
plotIons(data, ions = c(43, 54), RT = c(10,11), yscale, color, 
	plotGraph = TRUE, plotAbundantIons = TRUE, numberOfIons = c(1), 
	save = TRUE, folder, output = "fragments")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
when data is missing, a dialog box will pop up allowing the user to click-and-point to a CDF file. Alternatively, data can take a character string naming the path to the CDF file to be analysed or to a vector containing a xcmsRaw object (See ?xcms::xcmsRaw).
}
  \item{ions}{
numeric vector indicating the ion mass fragments to be highlighted, such as ions = c(106,58,60) for ion mass fragments 106 m/z, 58 m/z and 60 m/z or ions = c(70:73) for ion mass fragments 70 m/z, 71 m/z and 73 m/z. These ions will be highlighted using the colors specified through the argument colors.
}
  \item{RT}{
a numerical vector of size 2 indicating the retention time window to be analysed, such as RT = c(10,11) for retention time window from 10 minutes to 11 minutes.
}
  \item{yscale}{
a numerical vector of size 2 defining the scale of the Y axis of the chromatogram generated by plotIons. For example, if yscale = c(0,200), the chromatogram generated by plotIons will have its Y axis going from 0 to 200. If yscale is missing, 0 is set as the minimum value of Y axis while the maximum intensity of the total ion chromatogram (TIC) is set as the maximum value of the Y axis. 
}
  \item{color}{
a character vector, such as c("red", "blue", "#CDAA7D"), defining the colors to be used highlighting the ion mass fragments specified by the argument ions. If more ions than colors are defined, the colors defined will be recycled. If color is missing, the algorithm automatically selects colors from palette().
}
  \item{plotGraph}{
a logical vector (TRUE or FALSE) indicating if the chromatogram should be plotted. 
}
  \item{plotAbundantIons}{
a logical vector (TRUE or FALSE) indicating wether additional abundant ion mass fragments, other than the ion mass fragments specified through ions, should be highlighted in the chromatogram. It allows identifying those fragments most abundant during the retention time window analysed. 
}
  \item{numberOfIons}{
if plotAbundantIons = TRUE, numberOfIons receives a numerical vector indicating the number of additional ion mass fragments to be plotted. For example, if ions = c(106, 58), plotAbundantIons = TRUE and numberOfIons = 2, the ion mass fragments 106 m/z, 58 m/z and 2 others, the two most abundant ions, will be highlighted using different colors.   
}
  \item{save}{
A logical vector (TRUE or FALSE) indicating wether a CSV file containing the list of ion mass fragments analysed should be generated in the folder defined through folder.
}
  \item{folder}{
if save = TRUE and folder is missing, a dialog box will pop up allowing the user to click-and-point to the folder where the CSV file generated by plotIons must be saved. Alternativelly, folder may receive a character string naming the path to the folder or directory where the CSV file must be saved.
}
  \item{output}{
if save = TRUE, output receives a character string naming the CSV file produced by plotIons.   
}
}
\details{
For interactivelly identifying the spectrum associated to specific regions of the chromatogram generated by plotIons, simply click on the desired region of the chromatogram. A new graphical window will open showing the spectrum and the proportion between ion mass fragments. Clicking on a different region of the chromatogram updates the graphical window to the spectrum associated to this new region. For stopping the function, simply click on any region of the chromatogram where the Y axis is negative, such as any area bellow the X axis.   

}
\value{
plotIons returns a list containing two data frames: a data frame containing the intensity of all the fragments during the retention time analyzed; and a data frame containing the spectrum associated to the last region of the chromatogram clicked by the user.
}
\references{
Aggio, R., Mayor, A., Reade, S., Probert, C.S.J, & Ruggiero, K. (2014). Identifying and quantifying metabolites by scoring peaks of GC-MS data. BMC Bioinformatics, doi:10.1186/s12859-014-0374-2. 
}
\author{
Raphael Aggio
}
\seealso{
\code{\link{buildLib}}, \code{\link{extract_ion}}, \code{\link{filterCDF}}, \code{\link{filterCDFBatch}},\code{\link{filterTotalReport}}, \code{\link{matchSpectra}}, \code{\link{findNoise}}, \code{\link{matchSpectraFile}} 
}
\examples{
	##### Load Sample ####
	CDFfile <- unzip(system.file("extdata/130513_REF_SOL2_2_50_50_1.CDF.zip", package = "MetaBox"))
	### Plot all fragments highlighting the fragment 43 and the two most abundant fragments ###
	exampleTest <- plotIons(CDFfile, ions = c(43), RT = c(10,11), save = FALSE, 
		plotGraph = FALSE, plotAbundantIons = TRUE, numberOfIons = c(2))
	### Print results ####
	print(exampleTest)
	### Try again using plotGraph = TRUE. Click on a region of the chromatogram! ###
	#exampleTest <- plotIons(CDFfile, ions = c(43), RT = c(10,11), save = FALSE, 
	#	plotGraph = TRUE, plotAbundantIons = TRUE, numberOfIons = c(2))
	### Print results ####
	print(exampleTest)	
}